/* We now include stdlib.h as well, because we use malloc and free,
 * which are defined there.
 */
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
  int *j;

  /* The sizeof operator takes a type and returns the width of that type
   * in bytes on this system.  Thus, usually sizeof(int) is 4 (or 8 on
   * 64-bit machines), sizeof(char) is almost always 1, etc. */
   */
  printf("An integer is %d bytes.\n", sizeof(int));

  /* malloc() takes a size in bytes as argument, allocates that many
   * bytes on the heap, and returns the address of the allocated space.
   * If it fails, malloc returns NULL---you should check after every call.
   * The return type of malloc is void *, i.e. pointer to void.  void is
   * a special type with no values; you can not declare a variable of
   * type void, only pointers to it, which you can not dereference or
   * do arithmetic on since void has no size.  To use void *, which
   * is an arbitrary address, it must be cast to the appropriate type,
   * in this case, int *.
   */
  j = (int *) malloc(2 * sizeof(int));
  if (j == NULL)
  {
    fprintf(stderr, "Could not allocate memory for j.\n");
    exit(1);
  }

  printf("j=%p\n", j);

  j[0] = 1;
  j[1] = 2;
  printf("j[0]=%d j[1]=%d\n", j[0], j[1]);

  /* Any pointer you get from malloc MUST, at some point, be freed by
   * someone.  Else you have a memory leak which is a very insidious
   * and hard to track down but serious bug.
   *
   * ALWAYS FREE MEMORY YOU GOT FROM malloc!
   */
  free(j);

  return 0;
}
