#include <stdio.h>
#include <stdlib.h>

/* Define a structure with three elements, twelve bytes wide.   This will
 * be a doubly-circularly-linked list.
 */
struct list
{
  struct list *next;
  struct list *prev;
  int data;
};

/* Define the type "list" as shorthand for the type "struct list",
 * the structure we just defined.
 */
typedef struct list list;

/* In order to use a function or type, it must be declared BEFORE any
 * uses of it.  Here we declare three functions we will be using, but
 * without actually defining their bodies.
 */
void make_list(list *, int);
void insert_list(list *, list *);
void free_list(list *);

int main(int argc, char **argv)
{
  list *l = NULL;
  list *tmp = NULL;
  int i;

  for (i = 0; i < 10; ++i)
  {
    tmp = (list *) malloc(sizeof(list));
    if (tmp == NULL)
    {
      fprintf(stderr, "Could not allocate memory for element.\n");
      exit(1);
    }

    make_list(tmp, i);
    if (l == NULL)
      l = tmp;
    else
      insert_list(l, tmp);
  }

  /* The syntax foo.next gets the member named "next" from foo, if foo
   * is a struct with such a member.  If, instead, foo is a pointer
   * to such a structure, gettings its "next" element would required
   * dereferencing it first: (*foo).next but because this is done so much,
   * you can also use the shorthand foo->next.
   */
  tmp = l;
  do
  {
    printf("%d\n", tmp->data);
    tmp = tmp->next;
  } while (tmp != l);

  free_list(l);
  return 0;
}

/* We now define the functions we only declared before. */

void make_list(list *l, int data)
{
  if (l == NULL)
    return;

  l->data = data;
  l->next = l;
  l->prev = l;
}

void insert_list(list *into, list *from)
{
  if (into == NULL || into->prev == NULL || from == NULL)
    return;

  into->prev->next = from;
  from->prev = into->prev;
  from->next = into;
  into->prev = from;
}

void free_list(list *l)
{
  list *tmp = l;
  list *next = NULL;

  if (l == NULL)
    return;

  do
  {
    next = tmp->next;
    free(tmp);
    tmp = next;
  } while (tmp != l);
}
