/* Here we find new preprocessor directives; #define defines a symbol,
 * and #ifndef is like an if-block, which only allows the text up to
 * #endif to be included if the given symbol is not defined.  Thus,
 * this setup with ifndef-define-endif prevents this header file from
 * appearing more than once, even if it is included twice.
 */

#ifndef LINKED_H
#define LINKED_H

#include <stdio.h>
#include <stdlib.h>

struct list
{
  struct list *next;
  struct list *prev;
  int data;
};

typedef struct list list;

void make_list(list *l, int data)
{
  if (l == NULL)
    return;

  l->data = data;
  l->next = l;
  l->prev = l;
}

void insert_list(list *into, list *from)
{
  if (into == NULL || from == NULL)
    return;

  into->prev->next = from;
  from->prev = into->prev;
  from->next = into;
  into->prev = from;
}

void free_list(list *l)
{
  list *tmp = l;
  list *next = NULL;

  if (l == NULL)
    return;

  do
  {
    next = tmp->next;
    free(tmp);
    tmp = next;
  } while (tmp != l);
}

#endif /* LINKED_H */
