#ifndef LINKED3_H
#define LINKED3_H

#include <stdio.h>
#include <stdlib.h>

/* There is one small change here; instead of data being an int, it is
 * now a void *.  Remember that that's a pointer to arbitrary memory of
 * unknown type, so this list can hold heterogeneous data.  It is up to
 * the code using the list to keep track of the types of the elements
 * pointed to.  Note that free_list frees the elements, too, although
 * it would be possible to depend on the user to free the elements.
 */

struct list
{
  struct list *next;
  struct list *prev;
  void *data;
};

typedef struct list list;

void make_list(list *l, void *data)
{
  if (l == NULL)
    return;

  l->data = data;
  l->next = l;
  l->prev = l;
}

void insert_list(list *into, list *from)
{
  if (into == NULL || from == NULL)
    return;

  into->prev->next = from;
  from->prev = into->prev;
  from->next = into;
  into->prev = from;
}

void free_list(list *l)
{
  list *tmp = l;
  list *next = NULL;

  if (l == NULL)
    return;

  do
  {
    next = tmp->next;
    free(tmp->data);
    free(tmp);
    tmp = next;
  } while (tmp != l);
}

#endif /* LINKED3_H */
