/* This #include line is an instruction to the C preprocessor to find the
 * file "stdio.h" in the standard library directories on the system and
 * include it here.  We do this because stdio.h contains the declaration
 * of the printf() function, which we use below.
 */
#include <stdio.h>

/* The main() function is the entry point of a C program.  Its return
 * value is interpreted as the ultimate value of the program; by
 * convention, in UNIX, the program returns 0 on "success", whatever
 * that means for the program, and any other value on "failure".
 */
int main(int argc, char **argv)
{
  int i;  /* Here, i is an integer, i.e. 4 bytes on the stack. */
  int *j; /* j is a pointer to an integer, i.e. the address of some integer. */

  /* Set i and print it out. */
  i = 1;
  printf("i=%d\n", i);

  /* Use the & "address-of" operator to set j to the address of i; we
   * now say j "points to" i.  We then use the * "dereference" operator
   * to get the value to which j points and print it out.
   */
  j = &i;
  printf("j=%p *j=%d\n", j, *j);

  /* We can assign to *j, which changes the value of i indirectly,
   * since j currently points to i.
   */
  *j = 2;
  printf("i=%d\n", i);

  return 0;
}
